import Keycloak from 'keycloak-js';
import { useEffect, useRef, useState } from 'react';
import { useAppDispatch } from '../store';

import { authorized } from '../store/authSlice';

export const useAuth = () => {
  const keycloak = new Keycloak({
    // url: import.meta.env.VITE_KEYCLOAK_URL,
    // realm: import.meta.env.VITE_KEYCLOAK_REALM,
    // clientId: import.meta.env.VITE_KEYCLOAK_CLIENT,
    //TODO: remove this hardcode
    url: 'https://keycloak.graphpolaris.com',
    realm: 'graphpolaris',
    clientId: 'web',
  });

  const isRun = useRef(false);
  const [isLogin, setLogin] = useState(false);
  const dispatch = useAppDispatch();

  useEffect(() => {
    if (isRun.current) return;

    isRun.current = true;
    keycloak
      .init({
        onLoad: 'login-required',
        enableLogging: false,
        checkLoginIframe: false,
      })
      .then(async (isAuthenticated) => {
        // console.log("useAuth useEffect", isAuthenticated, keycloak.idTokenParsed);
        setLogin(isAuthenticated);

        // just for example here:
        // const profile = await getUserProfile();
        // console.log("useAuth useEffect profile", profile);

        await dispatch(
          authorized({
            // Info from https://www.keycloak.org/docs/latest/securing_apps/
            userID: keycloak.idTokenParsed.preferred_username ?? '',
            sessionID: keycloak.idTokenParsed.sid ?? '',
            accessToken: keycloak.token ?? '',
            authorized: true,
          })
        );
      })
      .catch((err) => {
        console.error('useAuth err', err);
        throw err;
      });
  }, []);

  const getUserProfile = async () => {
    return keycloak
      .loadUserProfile()
      .then(function (profile) {
        return profile;
        // console.info("user info", JSON.stringify(profile, null, "  "))
      })
      .catch(function () {
        alert('Failed to load user profile');
      });
  };

  return isLogin;
};

// export useAuth;
