import {
  setQuerybuilderNodes,
  store,
} from '@graphpolaris/shared/lib/data-access/store';
import Graph, { MultiGraph } from 'graphology';
import {
  Attributes as GAttributes,
  Attributes,
  SerializedGraph,
} from 'graphology-types';
import {
  EntityNodeAttributes,
  NodeAttributes,
  QueryGraph,
  RelationNodeAttributes,
} from './model';
import { XYPosition } from 'reactflow';

/** monospace fontsize table */
const widthPerFontsize = {
  6: 3.6167,
  7: 4.2167,
  10: 6.0167,
};

export type QueryMultiGraph = SerializedGraph<
  NodeAttributes | EntityNodeAttributes | RelationNodeAttributes,
  GAttributes,
  GAttributes
>;

export class QueryMultiGraphology extends MultiGraph<
  NodeAttributes | EntityNodeAttributes | RelationNodeAttributes,
  GAttributes,
  GAttributes
> {
  public addPill2Graphology(
    attributes: NodeAttributes | EntityNodeAttributes | RelationNodeAttributes,
    id: string = (Date.now() + Math.floor(Math.random() * 1000)).toString()
  ): string | null {
    const { type, name } = attributes;
    if (!type || !name) return null;
    let { x, y } = attributes;

    // Check if x and y are present, otherwise set them to 0
    if (!x) x = 0;
    if (!y) y = 0;

    // Get the width and height of a node
    const { width, height } = calcWidthHeightOfPill(attributes);

    // Add a node to the graphology object
    const nodeId = this.addNode(id, { ...attributes, x, y, width, height });

    // Set the new nodes in the query builder slice TODO: maybe remove for efficiency
    store.dispatch(setQuerybuilderNodes(this.export()));

    return nodeId;
  }
}

/** Calculates the width and height of a query builder pill.
 * DEPENDS ON STYLING, if styling changed, change this.
 */
export function calcWidthHeightOfPill(attributes: Attributes): {
  width: number;
  height: number;
} {
  const { type, name } = attributes;

  let w = 0;
  let h = 0;
  switch (type) {
    case 'entity': {
      // calculate width and height of entity pill
      w = Math.min(name.length, 20) * widthPerFontsize[10]; // for fontsize 10px

      const widthOfPillWithoutText = 42.1164; // WARNING: depends on styling
      w += widthOfPillWithoutText;
      h = 20;
      break;
    }
    case 'relation': {
      // calculate width and height of relation pill
      w = Math.min(name.length, 20) * widthPerFontsize[10]; // for fontsize 10px

      const widthOfPillWithoutText = 56.0666; // WARNING: depends on styling
      w += widthOfPillWithoutText;
      h = 20;
      break;
    }
    case 'attribute': {
      // calculate width and height of relation pill
      const pixelsPerChar = widthPerFontsize[6]; // for fontsize 10px
      w = name.length * pixelsPerChar;

      const { datatype, operator } = attributes;
      let value = attributes['value'];
      if (!datatype || !operator) return { width: 0, height: 0 };
      if (!value) value = '?';

      // Add width of operator
      w += operator.length * widthPerFontsize[7];
      // use a max of 10, because max-width is set to 10ch;
      w += Math.min(value.length, 10) * widthPerFontsize[6];

      const widthOfPillWithoutText = 25.6666; // WARNING: depends on styling
      w += widthOfPillWithoutText;
      h = 12;
      break;
    }
  }

  return { width: w, height: h };
}

/** Interface for x and y position of node */
export interface NodePosition extends XYPosition { }

/** Returns from-position of relation node */
export function RelationPosToFromEntityPos(position: XYPosition): NodePosition {
  return { x: position.x - 60, y: position.y - 40 };
}

/** Returns to-position of relation node */
export function RelationPosToToEntityPos(position: XYPosition): NodePosition {
  return { x: position.x + 400, y: position.y };
}

/** Default position; x=0, y=0 */
export const DefaultPosition = { x: 0, y: 0 };
