// import { handles } from '@graphpolaris/shared/lib/querybuilder/usecases';
import { useTheme } from '@mui/material';
import React, { useEffect } from 'react';
import { ReactFlow, Handle, Position } from 'reactflow';
import styles from './entitypill.module.scss';
import { EntityNode, Handles } from '../../../model';

/**
 * Component to render an entity flow element
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const EntityFlowElement = React.memo(({ data }: EntityNode) => {
  const theme = useTheme();
  // console.log('EntityPill', data);

  // TODO: Change flow element width when text overflows
  const animation = data.fadeIn ? styles.entityFade : '';

  return (
    <div
      className={`${styles.entity} ${animation} query_builder-entity`}
      style={{ backgroundColor: theme.palette.custom.elements.entityBase[0] }}
    >
      <Handle
        id={Handles.ToRelation}
        type="source"
        position={Position.Bottom}
        className={
          styles.ToRelationHandle +
          ' ' +
          (false ? styles.handleConnectedFill : '')
        }
      />
      <Handle
        id={Handles.ToAttribute}
        type="target"
        position={Position.Bottom}
        className={
          styles.ToAttributeHandle +
          ' ' +
          (false ? styles.handleConnectedFill : '')
        }
      />
      <Handle
        id={Handles.ReceiveFunction}
        type="target"
        position={Position.Bottom}
        className={
          styles.ReceiveFunctionHandle +
          ' ' +
          (false ? styles.handleConnectedFill : '')
        }
      />

      <div className={styles.entityWrapper}>
        <span className={styles.entitySpan}>{data.name}</span>
      </div>
    </div>
  );
});

EntityFlowElement.displayName = 'EntityFlowElement';

export default EntityFlowElement;
