/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { Node, Command, Edge, Graph, Coordinates } from './types';
import { calcAverageCoordinates } from './utils';
import * as d3 from 'd3';

export class AggregateCommand implements Command {
  /**
   * Groups nodes by a provided attribute
   * Computes average coordinates for new grouped node
   * Updates edge coordinates accordingly
   */
  private dimension: string;

  constructor(dimension: string) {
    this.dimension = dimension;
  }

  execute(data: Graph) {
    const graph: Graph = { ...data };
    graph.nodes = aggregateNodes(graph, this.dimension);
    graph.edges = aggregateEdges(graph);
    return graph;
  }
}

const aggregateNodes = (graph: Graph, dimension: string): Node[] => {
  const nodesByAttribute: Map<string, Node[]> = d3.group(
    graph.nodes,
    (node: Node) => eval(`node.${dimension}`)
  );

  const bundleByAttribute: Node[] = [];

  nodesByAttribute.forEach((group) => {
    const lat: number[] = [];
    const long: number[] = [];
    const locations: string[] = [];

    group.forEach((location: Node) => {
      const coordinates: Coordinates = location.attributes.coordinates;
      lat.push(coordinates[0]);
      long.push(coordinates[1]);
      locations.push(location.id);
    });

    const average: [number, number] = calcAverageCoordinates(lat, long);
    const name: string = eval(`group[0].${dimension}`);

    bundleByAttribute.push({
      id: name,
      attributes: {
        coordinates: average,
        locations: locations,
      },
    });
  });

  return bundleByAttribute;
};

const aggregateEdges = (data: Graph): Edge[] => {
  const locations: { [key: string]: string[] } = data.nodes.reduce(
    (locations, node) => {
      locations[node.id] = node.attributes.locations;
      return locations;
    },
    {}
  );

  const bundles: Edge[] = [];

  data.edges.forEach((edge: Edge) => {
    const from_location: string | undefined = Object.keys(locations).find(
      (state) => locations[state].includes(edge.from)
    );

    const to_location: string | undefined = Object.keys(locations).find(
      (state) => locations[state].includes(edge.to)
    );

    if (from_location && to_location) {
      if (from_location === to_location) return; // edge is within group

      const from_coordinates: Coordinates | undefined = data.nodes.find(
        (node: Node) => node.id === from_location
      )?.attributes.coordinates;

      const to_coordinates: Coordinates | undefined = data.nodes.find(
        (node: Node) => node.id === to_location
      )?.attributes.coordinates;

      const newEdge: Edge = {
        ...edge,
        from: from_location,
        to: to_location,
        from_coordinates: from_coordinates,
        to_coordinates: to_coordinates,
      };

      bundles.push(newEdge);
    }
  });

  return bundles;
};
