/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { Edge } from './types';

export const EdgeCrossings = (edges: Edge[]) => {
  /**
   * Custom implementation from greadability.js (https://github.com/rpgove/greadability/blob/master/greadability.js)
   * Improved it with a sweep line algorithm
   */
  let n_crossings = 0;

  edges.sort((a, b) => a.from_coordinates[0] - b.from_coordinates[0]);

  const activeSegments: Edge[] = [];

  for (let i = 0; i < edges.length; i++) {
    const edge: Edge = edges[i];

    let count = 0;
    while (count < activeSegments.length) {
      const segment: Edge = activeSegments[count];
      if (segment.to_coordinates[0] <= edge.from_coordinates[0]) {
        activeSegments.splice(count, 1);
      } else {
        if (doEdgesCross(edge, segment)) {
          n_crossings++;
        }
        count++;
      }
    }

    activeSegments.push(edge);
  }

  return n_crossings;
};

const doEdgesCross = (edge_one, edge_two) => {
  const from1 = edge_one.from_coordinates;
  const to1 = edge_one.to_coordinates;
  const from2 = edge_two.from_coordinates;
  const to2 = edge_two.to_coordinates;

  return linesCross([from1, to1], [from2, to2]);
};

// Inspiration: https://github.com/rpgove/greadability/blob/master/greadability.js

const direction = (pi, pj, pk) => {
  var p1 = [pk[0] - pi[0], pk[1] - pi[1]];
  var p2 = [pj[0] - pi[0], pj[1] - pi[1]];
  return p1[0] * p2[1] - p2[0] * p1[1];
};

const onSegment = (pi, pj, pk) => {
  return (
    Math.min(pi[0], pj[0]) <= pk[0] &&
    pk[0] <= Math.max(pi[0], pj[0]) &&
    Math.min(pi[1], pj[1]) <= pk[1] &&
    pk[1] <= Math.max(pi[1], pj[1])
  );
};

const linesCross = (line1, line2) => {
  var d1, d2, d3, d4;

  d1 = direction(line2[0], line2[1], line1[0]);
  d2 = direction(line2[0], line2[1], line1[1]);
  d3 = direction(line1[0], line1[1], line2[0]);
  d4 = direction(line1[0], line1[1], line2[1]);

  if (
    ((d1 > 0 && d2 < 0) || (d1 < 0 && d2 > 0)) &&
    ((d3 > 0 && d4 < 0) || (d3 < 0 && d4 > 0))
  ) {
    return true;
  } else if (d1 === 0 && onSegment(line2[0], line2[1], line1[0])) {
    return true;
  } else if (d2 === 0 && onSegment(line2[0], line2[1], line1[1])) {
    return true;
  } else if (d3 === 0 && onSegment(line1[0], line1[1], line2[0])) {
    return true;
  } else if (d4 === 0 && onSegment(line1[0], line1[1], line2[1])) {
    return true;
  }

  return false;
};
